/* Author: Denis Pyatkov, May 2008
A sample read/write module for the DDR RAM controller. Performs 2 writes followed by 2 reads.

*/

module ddr_read_write
(

	output wire [1:0] ddr_cmd_o, //indicates to controller whehther read or write is needed
	output wire [1:0] ba_o, //bank address
	output wire [2:0] burst_len_o,  // (number of words in burst) / 2 
	output wire [12:0] row_o, //row number
	output wire [7:0] column_o, //column number
	output wire test_done, //ends simulation, IS NOT NEEDED WHEN CONTROLLER IS ON FPGA

	input wire clk,
	input wire rst,
	input wire init_active, //lock for statemachine while RAM initialization is performed
	input wire ddr_ctrl_done //notifies this module when controller has returned to idle state
);


	assign burst_len_o = 3'h1; // two words for ddr

	reg [1:0] ba;
	reg [12:0] row;
	reg [7:0] column;
	
	assign ba_o = ba;
	assign row_o = row;
	assign column_o = column;
	
	// column/row/ba multiplexor
	wire mux_sel;
	
	// test two ddr locations
	always @(mux_sel)
		if (mux_sel == 1) 
			{ba, row, column} = {2'b00, 13'h5, 8'h2};
		else
			{ba, row, column} = {2'b01, 13'h17, 8'h31};
	
	
	
	// state           | spare  | tst_done |   ddr_cmd | mux_sel | 
	//-----------------------------------------------------------------
	// st_idle         |   000  |    0     |   00    |    0
	// st_wr1          |   000  |    0     |   10    |    0
	// st_wr1_wt_ndone |   001  |    0     |   00    |    0
	// st_wr1_wt_done  |   010  |    0     |   00    |    0
	// st_wr2          |   000  |    0     |   10    |    1
	// st_wr2_wt_ndone |   000  |    0     |   00    |    1
	// st_wr2_wt_done  |   001  |    0     |   00    |    1
	// st_rd1          |   000  |    0     |   11    |    0
	// st_rd1_wt_ndone |   011  |    0     |   00    |    0
	// st_rd1_wt_done  |   100  |    0     |   00    |    0
	// st_rd2          |   000  |    0     |   11    |    1
	// st_rd2_wt_ndone |   010  |    0     |   00    |    1
	// st_rd2_wt_done  |   011  |    0     |   00    |    1
	// st_stop         |   000  |    1     |   00    |    0


	parameter st_idle         =  7'b000_0000; // 0x00 - hex number of state for debugging in waveform viewer
	parameter st_wr1          =  7'b000_0100; // 0x04
	parameter st_wr1_wt_ndone =  7'b001_0000; // 0x10
	parameter st_wr1_wt_done  =  7'b010_0000; // 0x20
	parameter st_wr2          =  7'b000_0101; // 0x05
	parameter st_wr2_wt_ndone =  7'b000_0001; // 0x01
	parameter st_wr2_wt_done  =  7'b001_0001; // 0x11
	parameter st_rd1          =  7'b000_0110; // 0x06
	parameter st_rd1_wt_ndone =  7'b011_0000; // 0x30
	parameter st_rd1_wt_done  =  7'b100_0000; // 0x40
	parameter st_rd2          =  7'b000_0111; // 0x07
	parameter st_rd2_wt_ndone =  7'b010_0001; // 0x21
	parameter st_rd2_wt_done  =  7'b011_0001; // 0x31
	parameter st_stop         =  7'b000_1000; // 0x08

	reg [6:0] state, next;
	
	assign {test_done, ddr_cmd_o, mux_sel} = state[3:0];
	
	
	always @(posedge clk or posedge rst)
		if (rst) state <= st_idle;
		else     state <= next;
		
	always @(state or init_active or ddr_ctrl_done)
	case(state)
	st_idle:
		if (init_active)   next = st_idle;
		else               next = st_wr1;

	st_wr1:                next = st_wr1_wt_ndone;

	st_wr1_wt_ndone:
		if (ddr_ctrl_done) next = st_wr1_wt_ndone;
		else               next = st_wr1_wt_done;

	st_wr1_wt_done:
		if (ddr_ctrl_done) next = st_wr2;
		else               next = st_wr1_wt_done;

	st_wr2:                next = st_wr2_wt_ndone;

	st_wr2_wt_ndone:
		if (ddr_ctrl_done) next = st_wr2_wt_ndone;
		else               next = st_wr2_wt_done;

	st_wr2_wt_done:
		if (ddr_ctrl_done) next = st_rd1;
		else               next = st_wr2_wt_done;
		
	st_rd1:                next = st_rd1_wt_ndone;

	st_rd1_wt_ndone:
		if (ddr_ctrl_done) next = st_rd1_wt_ndone;
		else               next = st_rd1_wt_done;

	st_rd1_wt_done:
		if (ddr_ctrl_done) next = st_rd2;
		else               next = st_rd1_wt_done;

	st_rd2:                next = st_rd2_wt_ndone;
	
	st_rd2_wt_ndone:
		if (ddr_ctrl_done) next = st_rd2_wt_ndone;
		else               next = st_rd2_wt_done;
	
	st_rd2_wt_done:
		if (ddr_ctrl_done) next = st_stop;
		else               next = st_rd2_wt_done;
	
	st_stop: begin 
		next = st_stop;
	end
	
	default next = st_idle;

	endcase



endmodule
