//
// file : ddram_ctrl.v
//

`include  "defs.v"

//
// commands ({cpu_cmd, arb_cmd}) :
//   000 : CMD_IDLE
//   001 : CMD_REFRESH
//   010 : CMD_WR
//   011 : CMD_RD
//   1xx : CMD_EXT, addr_ext_i & cmd_rcw_i are used
//  


module ddram_ctrl
  (
   // arbiter interface
   input  wire  [1:0] arb_cmd_i,
   input  wire        cpu_cmd_i,  // 0 to 1 transition executes cpu cmd
   input  wire  [2:0] cmd_rcw_i,  // CMD_EXT ras, cas, we
   input  wire [10:0] cmd_addr_i, // CMD_EXT address
   input  wire [12:0] row_i,
   input  wire  [2:0] len_i,      // (length in columns) = 2*len_i 
   output wire        done_o,

   
   // data in/out read/write enable
   output wire        ren_o,
   output wire        wen_o,

   // ddram interface
   output wire        dts_o,  // data buffers tristate
   output wire [12:0] addr_o,
   output wire  [2:0] rcw_o,  // ras, cas, we (command)

   // system   
   input  wire        CLK,
   input  wire        RST
   
   );


    // state encodings
    
    parameter st_idle   = 12'b0000_0100_1111;  // 04f idle / done / addr = row
    //-------
    parameter st_extcmd = 12'b0000_0111_0111;  // 077 external command
    parameter st_extdn  = 12'b0010_0100_1111;  // 24f idle / done / addr = row
   //-------
    parameter st_w_act  = 12'b0000_0000_0011;  // 003 write activate
    parameter st_w_nop1 = 12'b0000_0001_0111;  // 017 single nop
    parameter st_w_nop2 = 12'b0010_0001_0111;  // 217 single nop
    parameter st_write  = 12'b0001_0001_0100;  // 114 write command
    parameter st_w_brst = 12'b0001_0000_0111;  // 107 write burst, nop's with ren_o
    //-------
    parameter st_r_act  = 12'b0010_0100_0011;  // 243 read activate
    parameter st_r_nop1 = 12'b0010_0101_0111;  // 257 single nop
    parameter st_r_nop2 = 12'b0000_0101_0111;  // 057 single nop
    parameter st_read   = 12'b0010_1101_0101;  // 2d5 read command
    parameter st_r_brst = 12'b0010_1100_0111;  // 2c7 read burst, nop's with wen_o
    parameter st_refr   = 12'b1000_0100_0001;  // 041 refresh
    parameter st_rfwtdn = 12'b1000_0100_0111;  // 041 wait rf done
    //-------
    parameter st_term   = 12'b0000_0100_0010;  // 042 //046 rd/wr burst terminate



    wire [2:0] cmd = {cpu_cmd_i, arb_cmd_i[1:0]};

    // dts_o must be delayed for 2 clk, to account for VI fifo read cycle
    // and output (IO PAD) register
    wire  dts;
    reg dts_r0, dts_r1;
    
    always @(posedge CLK or posedge RST)
    	if (RST) {dts_r1, dts_r0} <= 0;
    	else     {dts_r1, dts_r0} <= {dts_r0, dts};
    
    assign dts_o = dts_r1;
    	 
    // 3 clk wen_o delay for cas latency + 3 clk for control & data pad regs.
    wire       wen;
    reg wen_r4, wen_r3, wen_r2, wen_r1, wen_r0;
    always @(posedge CLK or posedge RST)
    	if (RST) {wen_r4, wen_r3, wen_r2, wen_r1, wen_r0} <= 0;
    	else     {wen_r4, wen_r3, wen_r2, wen_r1, wen_r0} <= {wen_r3, wen_r2, wen_r1, wen_r0, wen};
    
    assign wen_o = wen_r4;
    
   // address mux
    wire [1:0] amxs;  // address mux select


    wire [12:0] full_addr;

`ifdef AUTO_PRECHARGE
    assign      full_addr = {2'b00, 1'b1, 10'h000};  // start from the column 0
`else
    assign      full_addr = {2'b00, 1'b0, 10'h000};  // start from the column 0
`endif

    // FIXME add ce
    mux_4dir_reg #(13) addrmux 
      (
       .O  (addr_o),
       .I0 (row_i),              // row
       .I1 (full_addr),          // 0, A10, column
       .I2 (13'b0),              // NOT USED
       .I3 ({2'b0, cmd_addr_i}), // address for CMD_EXT
       .S  (amxs), 
       .CLK(CLK), 
       .RST(RST)
       );

    // ras, cas, we mux
    wire [2:0] rcw; // {ras, cas, we} from the state machine
    wire       rcwsel = amxs[1];
    mux_2dir_reg #(3, 3'b111) rcwmux 
      (.O({rcw_o}), .I0(rcw), .I1(cmd_rcw_i), .S(rcwsel), .CLK(CLK), .RST(RST));

    // register done
    wire done;
    reg_1stg #(1,0) donereg 
      (.O1(done_o), .I(done), .CLK(CLK), .RST(RST) ) ;

    // column counter (burst length)
    reg [2:0]  clmcnt;
    wire       clmdn;
    assign     clmdn = &(clmcnt[2:0]);
        
    always @(posedge CLK or posedge RST)
      if (RST)           clmcnt <= 0;
      else if (done)     clmcnt <= len_i + 1;
      else               clmcnt <= clmcnt - 1;

    wire       rf, rfdn;
    
    // delay for rfdn
    reg rf_r5, rf_r4, rf_r3, rf_r2, rf_r1, rf_r0;
    always @(posedge CLK or posedge RST)
      if (RST) { rf_r5, rf_r4, rf_r3, rf_r2, rf_r1, rf_r0  } <= 0;
      else     { rf_r5, rf_r4, rf_r3, rf_r2, rf_r1, rf_r0  } <= { rf_r4, rf_r3, rf_r2, rf_r1, rf_r0, rf };
  
  	assign rfdn = rf_r5;
  

    
    
    // FSM
    // state         | rf |spare |ren||wen|dts|amxs||done| rcw
    //-------------------------------------------------
    //  st_idle      |  0 |  00  | 0 || 0 | 1 | 00 || 1  | 111
    //  
    //  st_extcmd    |  0 |  00  | 0 || 0 | 1 | 11 || 0  | 111   
    //  st_extdn     |  0 |  01  | 0 || 0 | 1 | 00 || 1  | 111 // similar to idle
    //
    //  st_w_act     |  0 |  00  | 0 || 0 | 0 | 00 || 0  | 011 // rcw = active
    //  st_w_nop1    |  0 |  00  | 0 || 0 | 0 | 01 || 0  | 111
    //  st_w_nop2    |  0 |  01  | 0 || 0 | 0 | 01 || 0  | 111    
    //  st_write     |  0 |  00  | 1 || 0 | 0 | 01 || 0  | 100 // rcw = write
    //  st_w_brst    |  0 |  00  | 1 || 0 | 0 | 00 || 0  | 111
    //
    //  st_r_act     |  0 |  01  | 0 || 0 | 1 | 00 || 0  | 011 // rcw = active
    //  st_r_nop1    |  0 |  01  | 0 || 0 | 1 | 01 || 0  | 111
    //  st_r_nop2    |  0 |  00  | 0 || 0 | 1 | 01 || 0  | 111
    //  st_read      |  0 |  01  | 0 || 1 | 1 | 01 || 0  | 101 // rcw = read
    //  st_r_brst    |  0 |  01  | 0 || 1 | 1 | 00 || 0  | 111
    //
    //  st_refr      |  1 |  00  | 0 || 0 | 1 | 00 || 0  | 001 // rcw 
    //  st_rfwtdn    |  1 |  00  | 0 || 0 | 1 | 00 || 0  | 111
    //  
    //  st_term      |  0 |  00  | 0 || 0 | 1 | 00 || 0  | 010 /* 110 ? */
    //  st_nop2      |  0 |  11  | 0 || 0 | 1 | 01 || 0  | 111
    //
    //
 

   


    reg [11:0] state, next;
    
    wire [1:0] spare;

    assign     {rf, spare[1:0],  ren_o,/* */wen, dts, amxs[1:0],/* */done, rcw[2:0]} = state[11:0];
   
    always @(posedge CLK or posedge RST)
      if (RST) state <= st_idle;
      else     state <= next;
    
    always @(state or clmdn or cmd[2:0] or rfdn)
      case (state)
        
        st_idle:
          case (cmd[2:0])
            3'b000:  next = st_idle;  // no cmd
            3'b001:  next = st_refr;  // refresh
            3'b010:  next = st_w_act; // write
            3'b011:  next = st_r_act; // read
            3'b100:  next = st_extcmd; 
            3'b101:  next = st_extcmd;
            3'b110:  next = st_extcmd;
            3'b111:  next = st_extcmd;
          endcase // case(cmd[1:0])

        st_refr:     next = st_rfwtdn;
        st_rfwtdn:     
          case (rfdn)
            1'b0:    next = st_rfwtdn;
            1'b1:    next = st_idle;
          endcase // case(rfdn)
        
        st_extcmd:   next = st_extdn;
        
        st_extdn:
          case (cmd[2]) 
            1'b1:    next = st_extdn;
            1'b0:    next = st_idle;
          endcase 
        
        st_w_act:    next = st_w_nop1;
        st_w_nop1:   next = st_w_nop2;
        st_w_nop2:   next = st_write;
        st_write:
          case(clmdn)
            1'b0:    next = st_w_brst;
            1'b1:    next = st_term;

          endcase // case(clmdn)
        st_w_brst:
          case(clmdn)
            1'b0:    next = st_w_brst;
            1'b1:    next = st_term;

          endcase // case(clmdn)

        st_r_act:    next = st_r_nop1;
        st_r_nop1:   next = st_r_nop2;
        st_r_nop2:   next = st_read;
        st_read:
          case(clmdn)
            1'b0:    next = st_r_brst;
            1'b1:    next = st_term;

          endcase // case(clmdn)
        st_r_brst:
          case(clmdn)
            1'b0:    next = st_r_brst;
            1'b1:    next = st_term;

          endcase // case(clmdn)

        default:     next = st_idle;
        
      endcase // case(state)

    

endmodule // ddram_ctrl
